# This file is part of the M30W software.
# Copyright (C) 2012-2013 M30W developers.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

from __future__ import division

import weakref, os, time, tempfile, subprocess, threading
import wx
from wx import aui
from M30W.debug import debug, not_implemented
from M30W import runtime, costume, media
from M30W.config import config
from dialogs import error

IMAGE = 'imageeditor'
SOUND = 'soundeditor'
SCRIPT = 'scripteditor'
MAX_NAME_LEN = 10

welcome_text = \
"""
Welcome to M30W!

M30W is a program designed to allow fast developing of Scratch projects.
It uses a unique text syntax to allow typing of blocks rather than laggy
dragging them around.

M30W currently is in development process, and we haven't implemented
running scripts yet; Don't look for the green flag ;)
Editing scripts is working, but because we use kurt to parse scripts, 
current limitations apply:

- Take care with the "length of" block: strings aren't dropdowns, lists are

- length of [Hello!]      // string
- length of [list v]      // list

- Variable names (and possibly other values, such as broadcasts) can't:
  * contain special identifiers (like end, if, etc.)
  * have trailing whitespace
  * contain special characters, rather obviously: like any of []()<> or equals =
  * be named after a block, eg. a variable called "wait until"
"""


class ScriptEditor(wx.Panel):
    font = wx.Font(10, wx.FONTFAMILY_MODERN,
                             wx.FONTSTYLE_NORMAL,
                             wx.FONTWEIGHT_NORMAL)
    style = wx.TextAttr(font=font)
    style.SetTabs([25])
    # TODO: let the user config tab width (25 * chrs)

    def __init__(self, sprite, token, *args, **kwargs):
        wx.Panel.__init__(self, *args, **kwargs)
        self.sprite = weakref.proxy(sprite, self.DeleteMyself)
        self.token = token
        self.sizer = wx.BoxSizer(wx.VERTICAL)
        self.textCtrl = wx.TextCtrl(self,
                                    style=wx.TE_MULTILINE | wx.HSCROLL)
        self.textCtrl.SetFont(self.font)
        self.textCtrl.SetDefaultStyle(self.style)
        self.textCtrl.write(self.sprite.code)

        self.sizer.Add(self.textCtrl, 1, wx.EXPAND)
        self.SetSizerAndFit(self.sizer)
        self.Bind(wx.EVT_TEXT, self.Save, self.textCtrl)
        self.lastsave = time.time()

    def Save(self, event):
        # #Allows undo&redo
        # if time.time() - self.lastletter > 0.3:
            # self.undo.append(self.textCtrl.GetValue())
        self.lastsave = time.time()
        self.sprite.code = self.textCtrl.GetValue()

    @not_implemented
    def OnRedo(self):
        pass

    @not_implemented
    def OnUndo(self):
        pass

    def DeleteMyself(self, proxy):
        leftPanel = self.GetParent().GetParent()
        wx.CallAfter(leftPanel.RemovePageByToken, self.token)


class ImageEditor(wx.Panel):
    def __init__(self, sprite, parent):
        wx.Panel.__init__(self, parent)

        self.listBook = wx.Listbook(self, style=wx.LB_LEFT)
        self.sprite = sprite
        self.il = wx.ImageList(32, 32)
        self.names = []
        self.listBook.AssignImageList(self.il)
        self.Bind(wx.EVT_LIST_ITEM_FOCUSED,
                  self.OnItemFocused,
                  self.listBook.GetListView())

        self.toolbar = wx.ToolBar(self, style=wx.TB_HORIZONTAL)

        self.Bind(wx.EVT_MENU, self.OnDown,
                  self.toolbar.AddSimpleTool(wx.ID_DOWN,
                                    wx.ArtProvider_GetBitmap(wx.ART_GO_DOWN),
                                    shortHelpString="Move Down"))
        self.Bind(wx.EVT_MENU, self.OnUp,
                  self.toolbar.AddSimpleTool(wx.ID_UP,
                                    wx.ArtProvider_GetBitmap(wx.ART_GO_UP),
                                    shortHelpString="Move Up"))
        self.toolbar.AddSeparator()
        self.Bind(wx.EVT_MENU,
                  self.OnDelete,
                  self.toolbar.AddSimpleTool(wx.ID_DELETE,
                                    wx.ArtProvider.GetBitmap(wx.ART_CROSS_MARK),
                                    shortHelpString="Delete"))
        self.Bind(wx.EVT_MENU,
                  self.OnDuplicate,
                  self.toolbar.AddSimpleTool(wx.ID_ANY,
                                             media.get_icon('duplicate'),
                                             shortHelpString="Duplicate"))
        self.toolbar.AddSeparator()
        self.Bind(wx.EVT_MENU,
                  self.OnEdit,
                  self.toolbar.AddSimpleTool(wx.ID_EDIT,
                                             media.get_icon('edit'),
                                             shortHelpString="Edit"))
        self.Bind(wx.EVT_MENU,
                  self.OnRefresh,
                  self.toolbar.AddSimpleTool(wx.ID_REFRESH,
                                             media.get_icon('refresh'),
                                             shortHelpString="Refresh"))
        if not self.sprite.is_stage:
            self.toolbar.AddSeparator()
            self.Bind(wx.EVT_MENU,
                      self.OnCenter,
                      self.toolbar.AddSimpleTool(wx.ID_ANY,
                                                 media.get_icon('center'),
                                                 shortHelpString="Center"))      

        self.toolbar.Realize()

        # object because we imported costume
        self.UpdateList()

        self.sizer = wx.BoxSizer(wx.VERTICAL)
        self.sizer.Add(self.listBook, 1, wx.EXPAND)
        self.sizer.Add(self.toolbar, 0, wx.EXPAND)

        self.SetSizerAndFit(self.sizer)

    @staticmethod
    def ShortenName(name):
        if len(name) < MAX_NAME_LEN + 3: return name
        else: return name[:MAX_NAME_LEN - 3] + '...'

    def UpdateList(self):
        self.listBook.DeleteAllPages()
        for object in self.sprite.costumes:
            self.il.Add(object.get_thumbnail(32, costume.FORMAT_BITMAP))
            self.listBook.AddPage(ResourceViewer(self,
                                                 object,
                                                 not self.sprite.is_stage),
                         self.ShortenName(object.name),
                         select=True,
                         imageId=self.il.GetImageCount() - 1
                         )
        self.listBook.SetSelection(self.sprite.costume)

    def OnUp(self, event):
        index = self.listBook.GetSelection()
        costumes = self.sprite.costumes
        page = self.listBook.GetPage(index)
        self.listBook.RemovePage(index)
        name = self.ShortenName(costumes[index].name)
        self.listBook.InsertPage(index - 1, page, name, select=True,
                                 imageId=index)
        (costumes[index - 1], costumes[index]) = (costumes[index],
                                                  costumes[index - 1])

    def OnDown(self, event):
        index = self.listBook.GetSelection()
        costumes = self.sprite.costumes
        page = self.listBook.GetPage(index)
        self.listBook.RemovePage(index)
        name = self.ShortenName(costumes[index].name)
        self.listBook.InsertPage(index + 1, page, name, select=True,
                                 imageId=index)
        (costumes[index], costumes[index + 1]) = (costumes[index + 1],
                                                  costumes[index])
        print costumes

    def OnItemFocused(self, event):
        self.toolbar.EnableTool(wx.ID_DELETE, len(self.sprite.costumes) > 1)
        self.toolbar.EnableTool(wx.ID_UP, event.GetIndex() != 0)
        self.toolbar.EnableTool(wx.ID_DOWN,
                            event.GetIndex() < len(self.sprite.costumes) - 1)

    def OnDuplicate(self, event):
        selection = self.listBook.GetSelection()
        costume = self.sprite.costumes[selection].duplicate()
        names = map(lambda a: a.name, self.sprite.costumes)
        while (costume.name in names):
            costume = costume.duplicate()
        self.sprite.costumes.insert(selection + 1, costume)
        wx.CallAfter(self.UpdateList)

    def OnDelete(self, event):
        selection = self.listBook.GetSelection()
        self.sprite.costumes.pop(selection)
        self.listBook.DeletePage(selection)

    def OnEdit(self, event):
        selection = self.listBook.GetSelection()
        if config['IMAGE_EDITOR_PATH']:
            path = os.path.join(tempfile.gettempdir(),
                                '%s.png' % id(self.sprite.costumes[selection]))
            self.sprite.costumes[selection].save_to(path)
            subprocess.Popen([config['IMAGE_EDITOR_PATH'], path])
        else:
            error("Please configure editor!",
                  "You must configure your favorite image editor "
                  "befor using this feature!")

    def OnRefresh(self, event):
        selection = self.listBook.GetSelection()
        path = os.path.join(tempfile.gettempdir(),
                       '%s.png' % id(self.sprite.costumes[selection]))
        if os.path.isfile(path):
            self.sprite.costumes[selection].load_from(path)
        else:
            return
        def refresh():
            runtime.stage.Refresh()
            self.UpdateList()
            runtime.spritePanel.UpdateThumbnail(0 if self.sprite.is_stage
                                else runtime.get_sprites().index(self.sprite))
        wx.CallAfter(refresh)

    def OnCenter(self, event):
        self.listBook.GetCurrentPage().Center()
 
class ResourceViewer(wx.Panel):
    def __init__(self, parent, resource, show_grid=True):
        wx.Panel.__init__(self, parent, style=wx.FULL_REPAINT_ON_RESIZE)
        self.resource = weakref.proxy(resource)
        self.show_grid = show_grid

        self.sizer = wx.BoxSizer(wx.HORIZONTAL)

        if self.show_grid:
            self.sizer.AddSpacer(10, 10)
            self.xText = wx.StaticText(self, wx.ID_ANY, label="X:")
            self.sizer.Add(self.xText, 0, border=5,
                           flag=wx.ALIGN_BOTTOM | wx.BOTTOM)

            self.xCtrl = wx.SpinCtrl(self,
                                 wx.ID_ANY,
                                 initial=self.resource.center[0],
                                 min=0,
                                 max=self.resource.size[0] - 1)
            self.sizer.Add(self.xCtrl, 0, flag=wx.ALIGN_BOTTOM)

            self.sizer.AddSpacer(5, 5)

            self.yText = wx.StaticText(self, wx.ID_ANY, label="Y:")
            self.sizer.Add(self.yText, 0, border=5,
                           flag=wx.ALIGN_BOTTOM | wx.BOTTOM)

            self.yCtrl = wx.SpinCtrl(self,
                                 wx.ID_ANY,
                                 initial=self.resource.center[1],
                                 min=0,
                                 max=self.resource.size[1] - 1)
            self.sizer.Add(self.yCtrl, 0, flag=wx.ALIGN_BOTTOM)

            self.Bind(wx.EVT_SPINCTRL, self.OnSpin, self.xCtrl)
            self.Bind(wx.EVT_SPINCTRL, self.OnSpin, self.yCtrl)

        self.SetSizerAndFit(self.sizer)

        self.Bind(wx.EVT_PAINT, self.OnPaint, self)

    def OnPaint(self, event):
        # Note a VERY misleading bug when the panel is too big for the place in
        # the AUINoteBook - the costume begins to stretch over borders etc.
        size = [i - 20 for i in self.GetSize()]  # 10px padding
        size[1] -= 25 # Costume name
        if self.show_grid:
            size[1] -= self.xCtrl.GetSize()[1]

        image_prop = self.resource.size
        ratio = min(size[0] / image_prop[0], size[1] / image_prop[1])

        image_size = [int(ratio * i) for i in image_prop]
        image_pos = ((size[0] - image_size[0]) / 2 + 10,
                     (size[1] - image_size[1]) / 2 + 35)

        if image_size[0] < 1 or image_size[1] < 1:
            return event.Skip()

        dc = wx.PaintDC(self)
        dc.BeginDrawing()
        font = wx.Font(13, wx.FONTFAMILY_MODERN, wx.FONTSTYLE_NORMAL, wx.FONTWEIGHT_NORMAL)
        dc.SetFont(font)
        extent = dc.GetTextExtent(self.resource.name)
        dc.DrawText(self.resource.name,
                    (size[0] - extent[0] + 20) // 2,
                    10)
        dc.DrawRectangle(image_pos[0] - 1, image_pos[1] - 1,
                         image_size[0] + 2, image_size[1] + 2)
        dc.DrawBitmap(self.resource.get_resized_image(image_size,
                                                  costume.FORMAT_BITMAP),
                      *image_pos)

        if self.show_grid:
            dc.DrawLine(ratio * self.xCtrl.GetValue() + image_pos[0],
                        0,
                        ratio * self.xCtrl.GetValue() + image_pos[0],
                        size[1] + 20)

            dc.DrawLine(0,
                        ratio * self.yCtrl.GetValue() + image_pos[1],
                        size[0] + 20,
                        ratio * self.yCtrl.GetValue() + image_pos[1])
        dc.EndDrawing()

        event.Skip()

    def Center(self):
        self.xCtrl.SetValue(self.resource.size[0] // 2)
        self.yCtrl.SetValue(self.resource.size[1] // 2)
        self.OnSpin(None)

    def OnSpin(self, event):
        self.resource.center = self.xCtrl.GetValue(), self.yCtrl.GetValue()
        self.Refresh()


class LeftPanel(wx.Panel):
    def __init__(self, *args, **kwargs):
        wx.Panel.__init__(self, *args, **kwargs)
        runtime.leftPanel = self
        self.SetMinSize((480, 500))

        self.sizer = wx.BoxSizer(wx.HORIZONTAL)
        debug("Making notebook...", 1)
        self.noteBook = aui.AuiNotebook(self)
        welcome = wx.TextCtrl(self,
                            style=wx.TE_MULTILINE | wx.HSCROLL)
        welcome.write(welcome_text)
        welcome.SetFont(ScriptEditor.font)
        welcome.SetDefaultStyle(ScriptEditor.style)

        self.noteBook.AddPage(welcome, 'Welcome')
        self.welcomeOpen = True

        self.pages = []

        self.noteBook.SetMinSize((300, 450))
        debug("Done.", -1)
        self.sizer.Add(self.noteBook, 1, wx.EXPAND)
        self.SetSizerAndFit(self.sizer)
        self.SetMinSize(self.GetSize())

        self.Bind(aui.EVT_AUINOTEBOOK_PAGE_CLOSE,
                  self.OnPageClose, self.noteBook)
        # wxpython 2.9 compatibility
        if wx.VERSION[1] > 8:
            self.imageList = wx.ImageList(16, 16)
            self.noteBook.AssignImageList(self.imageList)


    def _NewPage(self, Window, title, bitmap, token):
        if self.welcomeOpen:
            self.noteBook.DeletePage(0)
            self.welcomeOpen = False
        try:
            self.noteBook.AddPage(Window, title, select=True, bitmap=bitmap)
        except TypeError:
            # API change on wxpython >= 2.9
            i = self.imageList.Add(bitmap)
            self.noteBook.AddPage(Window, title, select=True,
                                  imageId=self.imageList.GetImageCount() - 1)
            self.noteBook.SetPageImage(self.noteBook.GetPageCount() - 1, i)
        self.pages.append(token)

    def OpenScriptEditor(self, sprite):
        """OpenScriptEditor(sprite)

        Opens a new ScriptEditor tab for the given sprite.
        """
        if (sprite.name, SCRIPT) in self.pages:
            self.noteBook.SetSelection(self.pages.index((sprite.name, SCRIPT)))
            return
        self._NewPage(ScriptEditor(sprite, (sprite.name, SCRIPT), self.noteBook),
                      sprite.name, media.get_icon('script'),
                      (sprite.name, SCRIPT))

    def OpenImageEditor(self, sprite):
        """OpenImageEditor(sprite)

        Opens a new ScriptEditor tab for the given sprite.
        """
        if (sprite.name, IMAGE) in self.pages:
            self.noteBook.SetSelection(self.pages.index((sprite.name, IMAGE)))
            return
        self._NewPage(ImageEditor(sprite, self.noteBook),
                      sprite.name, media.get_icon('image'),
                      (sprite.name, IMAGE))

    @not_implemented
    def OpenSoundEditor(self, sprite):
        pass

    # Cleans self.imageList on wxpython >= 2.9
    def OnPageClose(self, event):
        if self.welcomeOpen:
            self.welcomeOpen = False
            return
        try:
            self.imageList.Remove(event.GetOldSelection())
        except AttributeError:
            pass
        self.pages.pop(event.GetOldSelection())

    def RemovePageByToken(self, token):
        index = self.pages.index(token)
        self.noteBook.DeletePage(index)

    def RefreshPages(self):
        debug('Deleting old pages...')
        try:
            old_selection = self.pages[self.noteBook.GetSelection()]
        # No open pages
        except IndexError:
            old_selection = None
        for _ in xrange(self.noteBook.GetPageCount()):
            self.noteBook.DeletePage(0)
        debug("Done.")

        debug("Recreating opened pages...")
        names = {sprite.name: sprite for sprite in
                 runtime.project.sprites + [runtime.project.stage]}
        functions = {SCRIPT: self.OpenScriptEditor,
                     IMAGE: self.OpenImageEditor,
                     SOUND: self.OpenSoundEditor}
        open_pages, self.pages = self.pages, []

        for name, type in open_pages:
            if name in names:
                functions[type](names[name])

        if old_selection in open_pages:
            self.noteBook.SetSelection(open_pages.index(old_selection))
        debug("Done.")



import re
from block import Block
from line import Line


def cleanup(code, sprite):
    """cleanup(code, sprite) -> cleaned up code

    Clean up a list of lines from comments etc."""
    #Not implemented in-line comments yet...
    code = [Line(index, sprite, line) for index, line
            in enumerate(code)
            if type(line) != Line]

    code = filter(lambda s: bool(s.replace(' ', '')), code)
    code = filter(lambda s: not s.replace(' ', '').startswith('#'), code)
    return code


class Script():
    def __init__(self, sprite, sourcecode, docleanup=True):
        """__init__(sprite, list of code lines) -> Script object

        Makes a new instance of Script object and compiles the script.
        You have to bind the script to an event using the executer module.
        """
        self.sprite = sprite

        self.code = sourcecode
        if docleanup:
            self.code = cleanup(self.code, self.sprite)

        def totree(code):
            copy = code[:] + ['']
            pattern = r'( *)'
            codeblock = []

            i = 0
            while i < len(copy):
                line = copy[i]
                #Using re group to find if it's deeper...
                if len(re.match(pattern, line).group(0)) > 0:
                    codeblock.append(copy.pop(i))
                else:
                    if codeblock:
                        copy.insert(i,
                                    Script(self.sprite,
                                           [line[4:] for line in codeblock],
                                    docleanup=False))
                        codeblock = []
                    i += 1
            return copy[:-1]

        self.code = totree(self.code)

        #Putting multiple C blocks of the same block together.
        for index, line in enumerate(self.code[:-3]):
            if (line.__class__ is Line and
                self.code[index + 1].__class__ is Script and
                self.code[index + 2].__class__ is Line and
                self.code[index + 3].__class__ is Script):
                if self.code[index + 2].startswith('\\'):
                    self.code[index] = (line + ' {script} ' +
                                   self.code.pop(index + 2)[1:])

        #Modifying the specs before c blocks to 'spec {script}' so they can
        #matched using re.
        for index, line in enumerate(self.code[:-1]):
            if type(line) is Line and self.code[index + 1].__class__ is Script:
                self.code[index] += ' {script}'

        self.sourcecode = self.code[:]

        #Converting Lines to Blocks.
        for index, line in enumerate(self.code):
            if line.__class__ == Script:
                continue
            cargs = []
            for i in range(index + 1, len(self.code)):
                if self.code[i].__class__ == Script:
                    cargs.append(self.code[i])
                else:
                    break
            self.code[index] = Block(line, cargs)

    def execute(self):
        """execute()

        Execute this script. This is actually executing it, not queuing it up.
        Returns when the script stops to run, not at all if it doesn't.
        """
        for block in self.code:
            block.execute()

    def print_code(self, code=None, indent=0):
        """print_code()

        Prints the sourcecode."""
        if not code:
            code = self.sourcecode

        printed_c = False
        for line in code:
            if line.__class__ == Script:
                if printed_c:
                    print ''
                line.print_code(indent=indent + 1)
                printed_c = True
            else:
                print indent * '\t' + line
                printed_c = False


if __name__ == '__main__':
    print '-' * 80 + "\nScript parsing test:\n"
    script = Script(None, r"""move 12 steps
forever
    if [1 > 2]
        say [32 + 32]
    \else
    # This is a comment.
    # In-line do not work (no idea how to check if it's inside
    # line or not, asking on AT)
        if ['42' = '42']
            say 'one is not bigger than two'
    repeat 4
        say 'That\'s a test'
""".split('\n'))
    script.print_code()
    print '-' * 80 + "\nLine objects test:\n"
    a = Line(42, 'a sprite', 'spam spam spam')
    print a, a.line, a.sprite, a.__class__
    b = a + ' eggs'
    print b, b.line, b.sprite, b.__class__



# This file is part of the M30W software.
# Copyright (C) 2012-2013 M30W developers.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

"""Contains the Costume class.
"""
import re
from PIL import Image
import wx
import os
try:
    import kurt
except ImportError:
    pass


FORMAT_PIL = Image.Image
FORMAT_BITMAP = wx.Bitmap
FORMAT_IMAGE = wx.Image
VALID_FORMATS = (FORMAT_PIL, FORMAT_BITMAP, FORMAT_IMAGE)


def __PIL_to_wxImage(image):
    output = wx.EmptyImage(image.size[0], image.size[1])
    output.SetData(image.convert("RGB").tostring())
    output.SetAlphaData(image.convert("RGBA").tostring()[3::4])
    return output


def __wxImage_to_PIL(image):
    if not image.HasAlpha():
        image.InitAlpha()  # Avoid binary transparency

    w, h = image.GetSize()
    data = image.GetData()

    redImage = Image.new("L", (w, h))
    redImage.fromstring(data[0::3])

    greenImage = Image.new("L", (w, h))
    greenImage.fromstring(data[1::3])

    blueImage = Image.new("L", (w, h))
    blueImage.fromstring(data[2::3])

    alphaImage = Image.new("L", (w, h))
    alphaImage.fromstring(image.GetAlphaData())
    pil = Image.merge('RGBA', (redImage,
                               greenImage, blueImage, alphaImage))
    return pil


def _convert(image, format):
    """convert(image, format)

    Converts the given image object to the specified format (FORMAT_*)
    """
    # Thanks http://jehiah.cz/a/pil-to-wxbitmap,
    # http://wxpython-users.1045709.n5.nabble.com/
    # Converting-PIL-images-to-wx-Image-and-back-td2353376.html

    assert format in VALID_FORMATS, 'Invalid format!'
    assert image.__class__ in VALID_FORMATS, 'Invalid image object!'

    # Eliminating all cases where we get the image with the wanted format
    if (isinstance(image, format)):
        return image

    # Converting to wx.Image
    if isinstance(image, FORMAT_PIL):
        output = __PIL_to_wxImage(image)

    elif isinstance(image, FORMAT_BITMAP):
        output = wx.ImageFromBitmap(image)

    else:
        output = image

    if format == FORMAT_IMAGE:
        return output
    elif format == FORMAT_PIL:
        return __wxImage_to_PIL(output)
    else:
        return wx.BitmapFromImage(output)


class Costume():
    def __init__(self, image, name, center=(None, None)):
        """Costume(image, name, center=(None, None) -> new Costume object

        :param image: path or image object
        :type image: wx.Image/Bitmap, PIL.Image, path
        :param name: the costume's name
        :type name: string
        :param center: the costume's center
        :type center: tuple (x, y). None if should be middle of the image
        """

        self.name = name

        if isinstance(image, str):
            self.load_from(image)
        elif image.__class__ in VALID_FORMATS:
            self.image = _convert(image, FORMAT_PIL)
        else:
            raise TypeError("'%s' object isn't a valid image object or path"
                            % image.__class__.__name__)

        self.center = [j if j != None else self.image.size[i] // 2 for
                       i, j in enumerate(center)]  # Because center can be 0 :/

    def __getstate__(self):
        dict = self.__dict__.copy()
        image = {'pixels': self.image.tostring(),
                 'size': self.image.size,
                 'mode': self.image.mode}
        dict['image'] = image
        return dict

    def __setstate__(self, dict):
        self.__dict__ = dict
        self.image = Image.fromstring(self.image['mode'],
                                      self.image['size'],
                                      self.image['pixels'])

    def __repr__(self):
        return "<Costume %s size=%s center=%s at %s>" % (self.name,
                                                         self.size,
                                                         self.center,
                                                         id(self))

    def duplicate(self):
        match = re.match("(.*?)\(([0-9]*)\)", self.name)
        if match:
            name = match.group(1) + " (%s)" % (int(match.group(2)) + 1)
        else:
            name = self.name + " (2)"

        return Costume(self.image, name, self.center)

    @classmethod
    def from_kurt(cls, image):
        return cls(image.get_image(), image.name, image.rotationCenter.value)

    def to_kurt(self):
        image = kurt.Image.from_image(self.name, self.image)
        image.rotationCenter = kurt.Point(*self.center)
        return image

    @property
    def size(self):
        return self.image.size

    @property
    def center(self):
        return self.center

    @center.setter
    def center(self, x=0, y=0):
        self.center = (x, y)

    def save_to(self, path, type=wx.BITMAP_TYPE_PNG):
        _convert(self.image, FORMAT_IMAGE).SaveFile(path, type)

    def load_from(self, path):
        try:
            self.image = Image.open(path)
            self.image.load()
        # decoder zip not available
        except IOError:
            self.image = _convert(wx.Image(path),
                                  FORMAT_PIL)

    def get_image(self, format=wx.Bitmap):
        """get_image(self, format=wx.Bitmap) -> image object in selected format

        Returns the image object of this costume in selected format
        :param format: return type
        :type format: PIL.Image.Image | wx.Image | wx.Bitmap
        """
        return _convert(self.image, format)

    def get_thumbnail(self, size, format=wx.Bitmap):
        try:
            copy = self.image.copy()
            copy.thumbnail((size, size))
            new = Image.new('RGBA', (size, size))
            new.paste(copy, tuple([(size - i) // 2 for i in copy.size]))
            return _convert(new, format)
        # Bug on 1x1 images using Imageops.fit(), not sure if still relevant
        except ZeroDivisionError:
            copy = _convert(self.image, format=wx.Image)
            pos = [(i - size) / 2 for i in copy.GetSize()]
            return _convert(copy.Resize((size, size), pos), format)

    def get_resized_image(self, size, format=wx.Bitmap):
        """get_resized_image(self, size, format=wx.Bitmap) -> resized image
        :param size: (new width, new length)
        :type size: tuple
        :param format: return type
        :type format: PIL.Image.Image | wx.Image | wx.Bitmap
        """
        return _convert(self.image.resize(size,
                                          resample=Image.ANTIALIAS),
                        format)

def get_default_costume():
    import media
    return Costume(media.get_icon('m30w'), 'New Costume')

def get_default_background():
    return Costume(Image.new('RGB', (480, 360), '#fff'), 'New Background')


class Line(str):
    """Line(line, sprite, string) -> Line object

    For error handling support, this is basically a wrapper for the str type.
    You can get the original line of that code line using l.line."""
    def __new__(cls, line, sprite, string):
        obj = str.__new__(cls, string)
        obj.line = line
        obj.sprite = sprite
        return obj

    def __add__(self, other):
        return self.__class__(self.line, self.sprite, str(self)[:] + other)

    def __getslice__(self, i, j):
        return self.__class__(self.line, self.sprite,
                              str(self)[i:j])



#This file is part of the M30W software.
#Copyright (C) 2012-2013 M30W developers.
#
#This program is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program. If not, see <http://www.gnu.org/licenses/>.

"""Provides global user settings.

:Usage:
- config: a dictionary, stores all config informations
- save(): forces save of the configurations to the disk
"""
import sys
import os
from json import load, dump


PATH = {'win32': lambda:
            os.path.join(os.environ['APPDATA'], 'M30W', 'M30W.json'),
        'cygwin': lambda: 
            os.path.join(os.environ['APPDATA'], 'M30W', 'M30W.json'),
        'linux2': lambda: os.path.expanduser('~/.config/M30W/M30W.json'),
        'darwin': lambda:
            os.path.expanduser('~/Library/Application Support/M30W/M30W.json')
        }[sys.platform]()

if not os.path.exists(PATH):
    try:
        if not os.path.isdir(os.path.dirname(PATH)):
            os.mkdir(os.path.dirname(PATH))
        open(PATH, 'w')
    except (OSError, IOError):
        raise SystemExit("Could not create config file at path '%s'" % PATH)

try:
    config = load(open(PATH))
except ValueError:
    config = {}

def save():
    """save()

    Saves configurations to the config file.
    """
    dump(config, open(PATH, 'w'))



#This file is part of the M30W software.
#Copyright (C) 2012-2013 M30W developers.
#
#This program is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program. If not, see <http://www.gnu.org/licenses/>.

import wx
from M30W.debug import debug
from M30W import runtime, media
from M30W.config import config 

debug("Initializing menus...", 1)
from .menu import menus, MenuBar, SaveBeforeClosing
debug("Done.", -1)
debug("Initializing right panel...", 1)
from .rightPanel import RightPanel
debug("Done.", -1)
debug("Initializing left panel...", 1)
from .leftPanel import LeftPanel
debug("Done.", -1)


class MainFrame(wx.Frame):
    def __init__(self, *args, **kwargs):
        wx.Frame.__init__(self, *args, **kwargs)
        runtime.mainFrame = self
        self.SetTitle('M30W')

        debug("Making menus...", 1)
        menubar = MenuBar(self, menus)
        self.SetMenuBar(menubar)
        debug("Done.", -1)

        debug('Making right panel...', 1)
        self.right_pane = RightPanel(self)
        debug("Done.", -1)

        debug("Making left panel...", 1)
        self.left_pane = LeftPanel(self)
        debug("Done.", -1)

        self.sizer = wx.BoxSizer(wx.HORIZONTAL)
        self.sizer.Add(self.left_pane, 1, flag=wx.EXPAND)
        self.sizer.Add(self.right_pane, 0, flag=wx.EXPAND | wx.LC_ICON)
        self.SetSizerAndFit(self.sizer)

        self.SetMinSize(self.GetSize())

        self.SetSize(config.get('START_SIZE'))
        self.Maximize(config.setdefault('START_MAXIMIZED', False))

        self.SetIcon(media.get_icon('M30W', wx.Icon, '.ico'))
        wx.GetApp().SetTopWindow(self)
        self.Bind(wx.EVT_CLOSE, self.OnClose, self)

    def OnClose(self, event):
        if SaveBeforeClosing(event) == wx.ID_CANCEL:
            return event.Veto()
        debug("Saving frame size...", 1)
        config['START_SIZE'] = tuple(self.GetSize())
        config['START_MAXIMIZED'] = self.IsMaximized()

        debug("Done.", -1)
        self.Destroy()



#!/usr/bin/python

# This file is part of the M30W software.
# Copyright (C) 2012-2013 M30W developers.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

# M30W follows PEP8 to increase readability. Why won't you?

from M30W.debug import debug
from M30W.GUI import show_app

def main():
    debug("Starting GUI...", 1)
    show_app()

if __name__ == '__main__':
    print "M30W Copyright (C) 2012 M30W developers."
    main()



#This file is part of the M30W software.
#Copyright (C) 2012-2013 M30W developers.
#
#This program is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program. If not, see <http://www.gnu.org/licenses/>.

"""Module providing simple debug tools
"""
import sys
import inspect
try:
    import kurt
except ImportError:
    kurt = None

DEBUG = '--debug' in sys.argv
_INDENT = 0


def debug(message, indent=0):
    """debug(message, indent=0)

    Prints message to stdout if --debug flag is on.
    Use indent to indicate opening and closing of tasks."""
    if not DEBUG: return
    global _INDENT
    if indent and indent < 0:
        _INDENT += indent
    print '\t' * _INDENT,
    print message
    if indent and indent > 0:
        _INDENT += indent


not_implemented_functions = []
class not_implemented():
    def __init__(self, func):
        self.func = func
        not_implemented_functions.append(func)

    def __call__(self, *args, **kwargs):
        print ("Called function %s in file %s which is yet not implemented."
               % (self.func.__name__, inspect.getsourcefile(self.func)))


class NoKurt(Exception):
    def __init__(self):
        Exception.__init__(self, "Saving or loading of .sb files isn't "
                                 "working without kurt. Please install it "
                                 "if you want to do so!")

def ensure_kurt():
    """
    Ensures kurt is available on system, raises error if not.
    """
    if not kurt:
        raise NoKurt

def print_not_implemented_functions():
    for func in not_implemented_functions:
        print "function %s in file %s" % (func.__name__,
                                          inspect.getsourcefile(func))


if __name__ == '__main__':
    print """M30W debug tools, released under GPL3.
Copyright (C) 2012 M30W developers.\n"""
    debug("Debugging...")



#This file is part of the M30W software.
#Copyright (C) 2012-2013 M30W developers.
#
#This program is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program. If not, see <http://www.gnu.org/licenses/>.

import wx
from wx.lib.filebrowsebutton import FileBrowseButton as BrowseButton
from M30W.config import config

class Item(wx.Panel):
    def __init__(self, parent, infos):
        wx.Panel.__init__(self, parent)
        self.sizer = wx.BoxSizer(wx.HORIZONTAL)
        self.token = infos['token']
        self.type = infos['type']

        {'path': self.MakePath}[infos['type']](infos)

        self.SetSizerAndFit(self.sizer)

    def __repr__(self):
        return "<Item token=%s type=%s at %s>" % (self.token,
                                                  self.type,
                                                  id(self))

    def MakePath(self, infos):
        self.button = BrowseButton(self,
                                   labelText=infos['label'],
                                   fileMask=infos['pattern'])
        self.button.SetValue(config.get(self.token, ""))
        self.button.textControl.SetEditable(False)
        self.sizer.Add(self.button, 1, flag=wx.EXPAND)
        def get():
            return self.button.GetValue()
        self.GetValue = get

    def GetValue(self):
        raise NotImplementedError

class PreferencesDialog(wx.Dialog):
    settings = \
    ({'token': "SCRATCH_PATH",
      'label': "Scratch' Location:",
      'type': "path",
      'pattern': "*.*"},
     {'token': "IMAGE_EDITOR_PATH",
      'label': "Image Editor's Path:",
      "type": "path",
      'pattern': "*.*"})

    def __init__(self, *args, **kwargs):
        wx.Dialog.__init__(self, *args, **kwargs)
        self.items = []
        self.sizer = wx.BoxSizer(wx.VERTICAL)
        for item in self.settings:
            self.items.append(Item(self, item))
            self.sizer.Add(self.items[-1])

        self.buttonsSizer = wx.BoxSizer(wx.HORIZONTAL)
        self.sizer.Add(self.buttonsSizer)

        self.cancel = wx.Button(self, wx.ID_CANCEL)
        self.apply = wx.Button(self, wx.ID_APPLY)

        self.buttonsSizer.AddStretchSpacer(2)
        self.buttonsSizer.Add(self.cancel, 1)
        self.buttonsSizer.Add(self.apply, 1)

        self.Bind(wx.EVT_BUTTON, lambda evt: self.Destroy(), self.cancel)
        self.Bind(wx.EVT_BUTTON, self.OnApply, self.apply)

        self.SetSizerAndFit(self.sizer)
        self.Layout()
        self.Show()

    def OnApply(self, evt):
        for item in self.items:
            config[item.token] = item.GetValue()
        self.Destroy()




#This file is part of the M30W software.
#Copyright (C) 2012-2013 M30W developers.
#
#This program is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program. If not, see <http://www.gnu.org/licenses/>.

"""Module to store objects at runtime.
Objects are either set to None if not yet created.
GUI parts are:
* mainFrame
* leftPanel
* rightPanel
* noteBook (the coding area)
* stage
* spritePanel
Non-GUI objects are:
* sprites
"""
mainFrame = leftPanel = rightPanel = noteBook = stage = spritePanel = None

#global project, sprites, Sprite, stage
from M30W.project import Project
from M30W.sprites import Sprite
project = Project()
project.changed = False

#Other modules should use the provided methods to change sprites.
def get_sprites():
    return project.sprites

def get_stage():
    return project.stage

def add(sprite):
    if sprite.name in map(lambda x: x.name, project.sprites):
        raise NameError("Sprite %s already registered!" % sprite.name)
    project.sprites.append(sprite)

def new():
    names = map(lambda x: x.name, project.sprites)
    for i in xrange(1000):
        if 'Sprite %s' % i in names: continue
        add(Sprite('Sprite %s' % i))
        return project.sprites[-1]

def delete(index):
    project.sprites.pop(index)



#This file is part of the M30W software.
#Copyright (C) 2012-2013 M30W developers.
#
#This program is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program. If not, see <http://www.gnu.org/licenses/>.


class Script:
    def __init__(self, blocks, sprite):
        self.blocks = blocks
        self.sprite = sprite

    def __iter__(self):
        return self.runner(self.blocks)

    @staticmethod
    def runner(blocks):
        for block in blocks:
            for _ in block():
                yield _

class Block:
    def __init__(self):
        pass

    @classmethod
    def from_kurt(cls, morph):
        pass


#This file is part of the M30W software.
#Copyright (C) 2012-2013 M30W developers.
#
#This program is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program. If not, see <http://www.gnu.org/licenses/>.

from collections import OrderedDict
from weakref import WeakKeyDictionary, proxy
import wx, dialogs
from M30W.debug import debug, not_implemented
from M30W import runtime
from M30W.sprites import rotmodes
import M30W.media as media
from wx.lib.mixins.listctrl import TextEditMixin, ListCtrlAutoWidthMixin

MAX_NAME_LEN = 10

class EditableListCtrl(wx.ListCtrl, TextEditMixin, ListCtrlAutoWidthMixin):
    def __init__(self, *args, **kwargs):
        wx.ListCtrl.__init__(self, *args, **kwargs)
        TextEditMixin.__init__(self)
        ListCtrlAutoWidthMixin.__init__(self)
        self._onResize_ = self._onResize
        self._doResize_ = self._doResize


class SpriteViewer(wx.Panel):
    sprite_attr = OrderedDict((('Name', 'name'),
              ('Position', 'pos'),
              ('Rotation', 'rotmode'),
              ('Direction', 'direction'),
              ('Draggable', 'draggable'),
              ('Visible', 'visible'),
              ('Volume', 'volume')))
    stage_attr = OrderedDict((('Name', 'name'),
             ('Volume', 'volume'),
             ('Tempo (BPM)', 'tempo')))

    def __init__(self, parent, sprite):
        super(SpriteViewer, self).__init__(parent)
        self.sprite = proxy(sprite, lambda proxy: self.Destroy())

        self.listCtrl = EditableListCtrl(self, style=wx.LC_REPORT |
                                wx.LC_NO_HEADER | wx.LC_VRULES | wx.LC_HRULES)
        self.listCtrl.InsertColumn(0, 'property')
        self.listCtrl.InsertColumn(1, 'value')
        if self.sprite.name == 'Stage':
            self.sprite_attr = self.stage_attr
        for label, attr in self.sprite_attr.iteritems():
            self.listCtrl.Append((label, getattr(self.sprite, attr)))
        self.listCtrl.SetColumnWidth(0, -1)

        self.sizer = wx.BoxSizer(wx.VERTICAL)
        self.sizer.Add(self.listCtrl, 1, wx.EXPAND)
        self.SetSizerAndFit(self.sizer)

        self.Bind(wx.EVT_LIST_BEGIN_LABEL_EDIT, self.OnBeginLabelEdit,
                  self.listCtrl)
        self.Bind(wx.EVT_LIST_END_LABEL_EDIT, self.OnEndLabelEdit,
                  self.listCtrl)

    def __repr__(self):
        return '<SpriteViewer for sprite "%s" at %s>' % (self.sprite.name,
                                                         id(self))

    def OnBeginLabelEdit(self, event):
        event.Allow()
        attr = self.sprite_attr[self.listCtrl.GetItem(event.GetIndex(),
                                                      0).GetText()]
        if event.m_col == 0: event.Veto()
        #Don't let Stage get renamed
        if self.sprite.name == 'Stage' and attr == 'name':
            event.Veto()
        if attr in self.specials:
            self.listCtrl.SetStringItem(event.GetIndex(), 1,
                                    str(self.specials[attr](event.GetLabel())))
            setattr(self.sprite, attr, self.specials[attr](event.GetLabel()))
            event.Veto()


    def OnEndLabelEdit(self, event):
        attr = self.sprite_attr[self.listCtrl.GetItem(event.GetIndex(),
                                                      0).GetText()]
        value = event.GetLabel()
        try:
            setattr(self.sprite, attr, value)
            self.listCtrl.SetStringItem(event.GetIndex(), 1,
                                        getattr(self.sprite, attr))
            if attr == 'name':
                wx.CallAfter(runtime.spritePanel.UpdateList)
        except (TypeError, ValueError):
            self.listCtrl.SetStringItem(event.GetIndex(), 1,
                                        str(getattr(self.sprite, attr)))
            event.Veto()

    specials = {'draggable': lambda t: False if t == 'True' else True,
                'visible': lambda t: False if t == 'True' else True,
                'rotmode':
            lambda t: rotmodes[(rotmodes.index(t) + 1) % 3]}


class SpritePanel(wx.Panel):
    def __init__(self, *args, **kwargs):
        wx.Panel.__init__(self, *args, **kwargs)
        runtime.spritePanel = self

        debug("Making sprites listbook...", 1)

        self.listBook = wx.Listbook(self, style=wx.LB_LEFT)
        self.il = wx.ImageList(32, 32)
        self.listBook.AssignImageList(self.il)
        #Stores refs to SpriteViewers and thumbnails
        self.last_update = WeakKeyDictionary()

        self.Bind(wx.EVT_LIST_ITEM_ACTIVATED, self.OnOpenScriptEditor,
                  self.listBook.GetListView())

        self.Bind(wx.EVT_LIST_ITEM_FOCUSED, self.OnItemFocused,
                  self.listBook.GetListView())

        debug("Done.")

        debug("Making Toolbar...")
        self.toolBar = wx.ToolBar(self, style=wx.TB_HORIZONTAL)
        self.Bind(wx.EVT_MENU, self.OnNew,
                  self.toolBar.AddSimpleTool(wx.ID_ANY,
                                wx.ArtProvider_GetBitmap(wx.ART_NEW),
                                shortHelpString="Make New Sprite"))
        self.Bind(wx.EVT_MENU, self.OnOpen,
                  self.toolBar.AddSimpleTool(wx.ID_ANY,
                                wx.ArtProvider_GetBitmap(wx.ART_FILE_OPEN),
                                shortHelpString="New Sprite From File"))
        wx.ID_EXPORT = wx.NewId()
        self.Bind(wx.EVT_MENU, self.OnExport,
                  self.toolBar.AddSimpleTool(wx.ID_EXPORT,
                                wx.ArtProvider_GetBitmap(wx.ART_FILE_SAVE),
                                shortHelpString="Export Sprite"))
        self.Bind(wx.EVT_MENU, self.OnDelete,
                  self.toolBar.AddSimpleTool(wx.ID_DELETE,
                                wx.ArtProvider_GetBitmap(wx.ART_CROSS_MARK),
                                shortHelpString="Delete Sprite"))
        self.Bind(wx.EVT_MENU, self.OnDown,
                  self.toolBar.AddSimpleTool(wx.ID_DOWN,
                                wx.ArtProvider_GetBitmap(wx.ART_GO_DOWN),
                                shortHelpString="Move Down"))
        self.Bind(wx.EVT_MENU, self.OnUp,
                  self.toolBar.AddSimpleTool(wx.ID_UP,
                                wx.ArtProvider_GetBitmap(wx.ART_GO_UP),
                                shortHelpString="Move Up"))

        self.toolBar.AddSeparator()

        self.Bind(wx.EVT_MENU,
                  self.OnOpenScriptEditor,
                  self.toolBar.AddSimpleTool(wx.ID_ANY,
                                             media.get_icon('script'),
                                             shortHelpString="Edit Scripts"))
        self.Bind(wx.EVT_MENU,
                  self.OnOpenImageEditor,
                  self.toolBar.AddSimpleTool(wx.ID_ANY,
                                             media.get_icon('image'),
                                             shortHelpString="Edit Costumes"))
        self.Bind(wx.EVT_MENU,
                  self.OnOpenSoundEditor,
                  self.toolBar.AddSimpleTool(wx.ID_ANY,
                                             media.get_icon('sound'),
                                             shortHelpString="Edit Sounds"))

        self.toolBar.Realize()
        debug("Done.")

        #Is here out of context, but is necessary so the up/down buttons
        #are disabled.
        self.UpdateList()

        self.sizer = wx.BoxSizer(wx.VERTICAL)
        self.sizer.Add(self.listBook, 1, flag=wx.EXPAND)
        self.sizer.Add(self.toolBar, 0, flag=wx.EXPAND)
        self.SetSizerAndFit(self.sizer)

    @staticmethod
    def GetItems():
        return [runtime.get_stage()] + runtime.get_sprites()

    @staticmethod
    def ShortenName(name):
        if len(name) < MAX_NAME_LEN: return name
        else: return name[:MAX_NAME_LEN - 3] + '...'

    def UpdateList(self, select=0):
        self.listBook.DeleteAllPages()

        debug("Creating new pages and putting them in...")
        for sprite in self.GetItems():
            self.il.Add(sprite.costumes[sprite.costume].get_thumbnail(32))
            self.listBook.AddPage(SpriteViewer(self.listBook, sprite),
                                  self.ShortenName(sprite.name),
                                  imageId=self.il.GetImageCount() - 1)
        debug("Done.")

    def UpdateThumbnail(self, index):
        sprite = self.GetItems()[index]
        self.il.Replace(index,
                        sprite.costumes[sprite.costume].get_thumbnail(32))

    def OnItemFocused(self, event):
        self.toolBar.EnableTool(wx.ID_UP,
                                event.GetIndex() >= 2)
        self.toolBar.EnableTool(wx.ID_DOWN,
                event.GetIndex() != 0 and
                event.GetIndex() < len(runtime.get_sprites()))
        self.toolBar.EnableTool(wx.ID_DELETE,
                event.GetIndex() != 0)
        self.toolBar.EnableTool(wx.ID_EXPORT,
                event.GetIndex() != 0)

    def OnOpenScriptEditor(self, event):
        runtime.leftPanel.OpenScriptEditor(
                            self.GetItems()[self.listBook.GetSelection()])

    def OnOpenImageEditor(self, event):
        runtime.leftPanel.OpenImageEditor(
                            self.GetItems()[self.listBook.GetSelection()])

    @not_implemented
    def OnOpenSoundEditor(self, event):
        pass

    def OnNew(self, event):
        sprite = runtime.new()
        self.il.Add(sprite.costumes[sprite.costume].get_thumbnail(32))
        self.listBook.AddPage(SpriteViewer(self.listBook, sprite),
                              self.ShortenName(sprite.name),
                              select=True,
                              imageId=self.il.GetImageCount() - 1)

    def OnOpen(self, event):
        path = dialogs.save("Please choose a sprite:",
                            "Scratch sprite files (*.sprite)|*.sprite|"
                            "All Files (*.*)|*.*")
        if path:
            runtime.project.new_sprite_from(path)

    def OnExport(self, event):
        path = dialogs.save("Please choose a sprite:",
                            "Scratch sprite files (*.sprite)|*.sprite|"
                            "All Files (*.*)|*.*")
        if path:
            sprite = runtime.project.sprites[self.listBook.GetSelection() - 1]
            sprite.save_to(path)

    def OnDelete(self, event):
        index = self.listBook.GetSelection()
        self.listBook.DeletePage(index)
        runtime.delete(index - 1)

    def OnDown(self, event):
        index = self.listBook.GetSelection()
        sprites = runtime.get_sprites()
        page = self.listBook.GetPage(index)
        self.listBook.RemovePage(index)
        #index + 1 because we have the stage also...
        name = self.ShortenName(runtime.get_sprites()[index - 1].name)
        self.listBook.InsertPage(index + 1, page, name, select=True,
                                 imageId=index)
        (sprites[index], sprites[index - 1]) = (sprites[index - 1],
                                                sprites[index])

    def OnUp(self, event):
        index = self.listBook.GetSelection()
        sprites = runtime.get_sprites()
        page = self.listBook.GetPage(index)
        self.listBook.RemovePage(index)
        #index + 1 because we have the stage also...
        name = self.ShortenName(runtime.get_sprites()[index - 1].name)
        self.listBook.InsertPage(index - 1, page, name, select=True,
                                 imageId=index)
        (sprites[index - 2], sprites[index - 1]) = (sprites[index - 1],
                                                    sprites[index - 2])



#This file is part of the M30W software.
#Copyright (C) 2012-2013 M30W developers.
#
#This program is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program. If not, see <http://www.gnu.org/licenses/>.

import time
from M30W.debug import debug, not_implemented
try:
    import kurt
except ImportError:
    pass

class Base(object):
    kwords = {'costumes': lambda: [],
              'costume': lambda: 0,
              'code': lambda: "",
              'vars': lambda: {},
              'lists': lambda: {},
              'volume': lambda: 100}

    def __init__(self, name, **kwargs):
        """Base(self, name, costumes=[], costume=0, code="", vars={}, lists={},
                volume=100)

        :Parameters:
        - `name`, `costumes`,`costume`, `code`, `vars`,
          `lists`, `volume`
        """
        debug("Creating sprite %s" % name)
        self._name = name

        # Functions because shared mutable objects
        for kword in self.kwords:
            if kword in kwargs: setattr(self, '_' + kword, kwargs[kword])
            else: setattr(self, '_' + kword, self.kwords[kword]())

        self._sounds_backup = kwargs['sounds'] if 'sounds' in kwargs else None

    def __repr__(self):
        return "<Sprite '%s' at %s>" % (self.name, id(self))

    @classmethod
    def from_kurt(cls, morph):
        raise NotImplementedError

    def to_kurt(self):
        raise NotImplementedError
    
    #Called from children, to reduce code be rewritten.
    def set_kurt_attrs(self, morph):
        morph.images = [costume.to_kurt() for costume in self._costumes]
        morph.costume = morph.images[self._costume]
        lists = {}
        for name, list in self._lists.iteritems():
            lists[name] = kurt.ScratchListMorph(name=name, items=list)
        morph.lists = lists
        morph.vars = self._vars.copy()  # I don't trust kurt :)
        morph.volume = self._volume

        if self._sounds_backup:
            morph.sounds = self._sounds_backup

        code = unicode(self.code).split('\n' * 3)
        code = [i for i in code if i.strip("\r\n ")]
        for sheet in code:
            sheet = kurt.parse_block_plugin(sheet)
            sheet.morph = morph
            sheet.pos = kurt.Point([20, 20])
            morph.scripts.extend([sheet])

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, value):
        self._name = value

    @property
    def code(self):
        return self._code

    @code.setter
    def code(self, value):
        self._code = value

    @property
    def vars(self):
        return self._vars

    @property
    def lists(self):
        return self._lists

    @property
    def costumes(self):
        return self._costumes

    @property
    def costume(self):
        return self._costume

    @costume.setter
    def costume(self, value):
        self.costumes[value]  # Validate that index is in range
        self._custume = value

    @property
    def active_costume(self):
        return self.costumes[self.costume]

    @active_costume.setter
    def active_costume(self, value):
        self.costume = self.costumes.index(value)

    @property
    def volume(self):
        return self._volume

    @volume.setter
    def volume(self, value):
        value = float(value)
        if not 0 <= value <= 100:
            print value
            raise TypeError("Value must bew in range 0-100")
        self._volume = value

    @property
    def is_stage(self):
        return self.name == 'Stage'

    # Sounds support comes later.

    #==========================================================================
    # Begin blocks code
    #==========================================================================
    @not_implemented
    def forever(self, args, cargs):
        pass

    @not_implemented
    def repeat(self, args, cargs):
        pass

    @not_implemented
    def dountil(self, args, crags):
        pass

    @not_implemented
    def doif(self, args, cargs):
        pass

    @not_implemented
    def doifelse(self, args, crags):
        pass

    def wait(self, args):
        exec_time = time.time()
        while not exec_time - time.time() > args[0]:
            yield

    @not_implemented
    def waituntil(self, args):
        pass



#This file is part of the M30W software.
#Copyright (C) 2012-2013 M30W developers.
#
#This program is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program. If not, see <http://www.gnu.org/licenses/>.

import sys
import os
import subprocess
import wx
from wx.lib.dialogs import messageDialog as message
from M30W.debug import debug, print_not_implemented_functions
from errorConsole import ErrorConsole
from preferences import PreferencesDialog
import dialogs
from M30W import media, runtime, IO
from M30W.config import config


SEPARATOR = 'separator'
SUBMENU = 'submenu'


class Menu(wx.Menu):
    def __init__(self, parent, items, *args, **kwargs):
        wx.Menu.__init__(self, *args, **kwargs)
        self.parent = parent
        for item, itemid, callback in items:
            if itemid == SEPARATOR:
                self.AppendSeparator()
            elif itemid == SUBMENU:
                self.AppendSubMenu()
            else:
                item = self.Append(itemid, item)
                self.parent.Bind(wx.EVT_MENU, callback, item)


class MenuBar(wx.MenuBar):
    def __init__(self, parent, menus, *args, **kwargs):
        wx.MenuBar.__init__(self, *args, **kwargs)
        self.parent = parent
        global menubar
        menubar = self
        for title, items in menus:
            self.Append(Menu(self.parent, items), title)

def SaveBeforeClosing(e):
    if not runtime.project.changed:
        return wx.ID_NO
    result = message(message='If you don''t save, changes will be lost.',
                    title='Save changes before closing?',
                    aStyle=wx.YES_NO | wx.YES_DEFAULT | wx.CANCEL).returned
    if result == wx.ID_YES:
        OnSave(e)
    return result

def OnNew(e):
    if SaveBeforeClosing(e) == wx.ID_CANCEL:
        return
    IO.new()

def OnOpen(e):
    if SaveBeforeClosing(e) == wx.ID_CANCEL:
        return
    path = dialogs.open("Please Choose File",
                        "Scratch 1.4 project file (*.sb)|*.sb"
                        "|All files (*.*)|*.*")
    if path:
        IO.open(path)

def OnSave(e):
    try:
        IO.save()
    except (ValueError, AttributeError):
        OnSaveAs(e)

def OnSaveAs(e):
    path = dialogs.save("Please Choose File",
                        "Scratch 1.4 project file (*.sb)|*.sb"
                        "|All files (*.*)|*.*")
    if path:
        IO.save_as(path)
        return True

def OnQuit(e):
    wx.GetApp().GetTopWindow().Close()

def default_scratch_path():
    if sys.platform == 'win32':
        if os.path.isfile(r'C:\Program Files\Scratch\Scratch.exe'):
            return r'C:\Program Files\Scratch\Scratch.exe'
        elif os.path.isfile(r'C:\Program Files (x86)\Scratch\Scratch.exe'):
            return r'C:\Program Files (x86)\Scratch\Scratch.exe'
    elif sys.platform == 'linux2':
        if os.path.isfile('/usr/bin/scratch'):
            return '/usr/bin/scratch'
    elif sys.platform == 'darwin':
        if os.path.isfile('/Applications/Scratch 1.4/Scratch.app'):
            return '/Applications/Scratch 1.4/Scratch.app'
    return None

def OnSaveAndOpenInScratch(e):
    OnSave(e)

    #Assuring event was not canceled from save as dialog
    if not runtime.project.path: return

    path = config.setdefault('SCRATCH_PATH', default_scratch_path())
    if not path:
        path = dialogs.open("Please Choose The Scratch Executable")
        if path:
            config['SCRATCH_PATH'] = path
        else:
            return

    if sys.platform == 'win32':
        subprocess.Popen([path, os.path.join(os.path.split(path)[0],
                                             'Scratch.image'),
                          runtime.project.path])
    else:
        subprocess.Popen([path, runtime.project.path])

def OnReload(e):
    if not runtime.project.path: return
    IO.reload()

def OnPreferences(event):
    PreferencesDialog(runtime.mainFrame)

def OnAbout(e):
    description = """M30W is a program designed to allow fast
developing of Scratch projects.
It uses a unique text syntax to allow typing of
blocks rather than laggy dragging them around."""
    mlicense = """M30W is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see http://www.gnu.org/licenses/"""
    info = wx.AboutDialogInfo()

    info.SetIcon(media.get_icon('M30W', wx.Icon, '.ico'))
    info.SetName('M30W')
    info.SetVersion('0.2.0')
    info.SetDescription(description)
    info.SetCopyright('(C) 2012 M30W developers')
    info.SetWebSite('http://scratch.mit.edu/forums/viewtopic.php?id=106225')
    info.SetLicence(mlicense)

    wx.AboutBox(info)

def OnConsole(event):
    sys.stdout = ErrorConsole(None)

def OnPrint(event):
    print_not_implemented_functions()


File = (('&New\tCtrl+N', wx.ID_NEW, OnNew),
        ('&Open\tCtrl+O', wx.ID_OPEN, OnOpen),
        (None, SEPARATOR, None),
        ('&Save\tCtrl+S', wx.ID_SAVE, OnSave),
        ('Save &As\tCtrl+Shift+S', wx.ID_SAVEAS, OnSaveAs),
        (None, SEPARATOR, None),
        ('&Quit', wx.ID_EXIT, OnQuit))
Edit = (('Save And Open In Scratch', wx.ID_ANY, OnSaveAndOpenInScratch),
        ('Reload', wx.ID_ANY, OnReload),
        (None, SEPARATOR, None),
        ("&Preferences", wx.ID_PREFERENCES, OnPreferences))
Help = [('Error Console', wx.ID_ANY, OnConsole),
        ('Print Not Implemented Functions', wx.ID_ANY, OnPrint),
        (None, SEPARATOR, None),
        ('About M30W', wx.ID_ABOUT, OnAbout)]
menus = (('&File', File), ('&Edit', Edit), ('&Help', Help))



#This file is part of the M30W software.
#Copyright (C) 2012-2013 M30W developers.
#
#This program is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program. If not, see <http://www.gnu.org/licenses/>.

import sys
import wx


class ErrorConsole(wx.Frame):
    def __init__(self, *args, **kwargs):
        wx.Frame.__init__(self, *args, **kwargs)
        self.SetMinSize((500, 400))
        self.SetTitle("Error Console")
        self.panel = wx.Panel(self)
        self.panel.sizer = wx.BoxSizer(wx.HORIZONTAL)
        self.textfield = wx.TextCtrl(self.panel,
                                     style=wx.TE_MULTILINE | wx.HSCROLL)
        self.textfield.Disable()
        self.panel.sizer.Add(self.textfield, 1, wx.EXPAND)
        self.panel.SetSizerAndFit(self.panel.sizer)
        self.Bind(wx.EVT_CLOSE, self.OnClose, self)
        self.Show(True)

    def OnClose(self, event):
        sys.stdout = sys.__stdout__
        event.Skip()

    def write(self, string):
        self.textfield.write(string)



#This file is part of the M30W software.
#Copyright (C) 2013 M30W developers.
#
#This program is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program. If not, see <http://www.gnu.org/licenses/>.

import wx

def inform(caption="", msg=""):
    dialog = wx.MessageDialog(None, message=msg, caption=caption,
                              style=wx.OK)
    dialog.ShowModal()
    dialog.Destroy()

def warn(caption="", msg=""):
    dialog = wx.MessageDialog(None, message=msg, caption=caption,
                              style=wx.OK | wx.ICON_EXCLAMATION)
    dialog.ShowModal()
    dialog.Destroy()

def error(caption="", msg=""):
    dialog = wx.MessageDialog(None, message=msg, caption=caption,
                              style=wx.OK | wx.ICON_ERROR)
    dialog.ShowModal()
    dialog.Destroy()

def ask(caption="", msg=""):
    dialog = wx.MessageDialog(None, message=msg, caption=caption,
                              style=wx.ICON_QUESTION | wx.YES_NO)
    value = dialog.ShowModal()
    dialog.Destroy()
    return value == wx.ID_YES

def open(msg="", wildcard="All files (*.*)|*.*"):
    dialog = wx.FileDialog(None, message=msg, wildcard=wildcard,
                           style=wx.OPEN)
    value = None
    if dialog.ShowModal() == wx.ID_OK:
        value = dialog.GetPath()
    dialog.Destroy()
    return value

def save(msg="", wildcard="All diles (*.*)|*.*"):
    dialog = wx.FileDialog(None, message=msg, wildcard=wildcard,
                           style=wx.SAVE)
    value = None
    if dialog.ShowModal() == wx.ID_OK:
        value = dialog.GetPath()
    dialog.Destroy()
    return value



#This file is part of the M30W software.
#Copyright (C) 2012-2013 M30W developers.
#
#This program is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program. If not, see <http://www.gnu.org/licenses/>.

from .base import Base
from M30W.costume import Costume, get_default_background
from M30W.lists import ScratchList
try:
    import kurt
except ImportError:
    pass


class Stage(Base):
    kwords = Base.kwords.copy()
    kwords.update({'tempo': lambda: 60})

    def __init__(self, **kwargs):
        """Stage(self, costumes=[], costume=0, code="", vars={}, lists={},
                volume=100, tempo=60)

        :Parameters:
        - `costumes`,`costume`, `code`, `vars`, `lists`, `volume`, `tempo`
        """
        Base.__init__(self, 'Stage', **kwargs)

        if not self.costumes:
            self.costumes.append(get_default_background())

    @classmethod
    def from_kurt(cls, morph):
        code = ("\n" * 3).join([script.to_block_plugin()
                                    for script in morph.scripts])

        costumes = [Costume.from_kurt(costume) for
                    costume in morph.backgrounds]
        costume = morph.backgrounds.index(morph.background)

        lists = {name: ScratchList(*list.items) for
                 name, list in morph.lists.iteritems()}

        if morph.sounds:
            sounds = morph.sounds
        else:
            sounds = None

        return cls(costumes=costumes,
                   costume=costume,
                   code=code,
                   vars=morph.variables.copy(),
                   lists=lists,
                   volume=morph.volume,
                   tempo=morph.tempoBPM,
                   sounds=sounds)


    def to_kurt(self):
        morph = kurt.Stage()
        self.set_kurt_attrs(morph)
        morph.tempoBPM = self._tempo
        return morph

    @property
    def tempo(self):
        return self._tempo

    @tempo.setter
    def tempo(self, value):
        self._tempo = value



#This file is part of the M30W software.
#Copyright (C) 2012-2013 M30W developers.
#
#This program is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program. If not, see <http://www.gnu.org/licenses/>.

from .base import Base
from M30W.costume import Costume, get_default_costume
from M30W.lists import ScratchList
from M30W.debug import debug, not_implemented, ensure_kurt
try:
    import kurt
except ImportError:
    pass


NORMAL = 'normal'
RL = "leftRight"
NO = "none"


class Sprite(Base):
    kwords = Base.kwords.copy()
    kwords.update({'x': lambda: 0.0,
                   'y': lambda: 0.0,
                   'direction': lambda: 90.0,
                   'rotmode': lambda: NORMAL,
                   'draggable': lambda: False,
                   'size': lambda: 100.0,
                   'visible': lambda: True})

    def __init__(self, name, **kwargs):
        super(Sprite, self).__init__(name, **kwargs)

        if not self.costumes:
            self.costumes.append(get_default_costume())
        self.redraw()

    @classmethod
    def from_kurt(cls, morph):
        costumes = [Costume.from_kurt(costume) for costume in morph.costumes]
        costume = morph.costumes.index(morph.costume)
        x, y, _, _ = morph.bounds.value
        y *= -1
        rx, ry = costumes[costume].center
        x += rx - 240
        y += -ry + 180
        code = ("\n" * 3).join([script.to_block_plugin()
                                for script in morph.scripts])
        rotmode = morph.rotationStyle.value
        direction = (morph.rotationDegrees + 90.0) % 360
        if direction > 180: direction -= 360
        lists = {name: ScratchList(*list.items) for
                 name, list in morph.lists.iteritems()}

        if morph.sounds:
            sounds = morph.sounds
        else:
            sounds = None

        return cls(morph.name,
                   costumes=costumes,
                   costume=costume,
                   code=code,
                   vars=morph.variables.copy(),
                   lists=lists,
                   volume=morph.volume,
                   x=x,
                   y=y,
                   direction=direction,
                   rotmode=rotmode,
                   draggable=morph.draggable,
                   size=morph.scalePoint.value[0] * 100,
                   visible=not morph.flags,
                   sounds=None)


    def to_kurt(self):
        morph = kurt.Sprite()
        self.set_kurt_attrs(morph)
        morph.name = self.name
        #Visibility is saved in morph.flags
        morph.flags = 0 if self.visible else 1

        morph.rotationStyle = kurt.Symbol(self.rotmode)
        direction = self.direction
        if direction < 0: direction += 360
        direction = (direction - 90) % 360
        morph.rotationDegrees = direction
        morph.scalePoint = kurt.Point((self.size / 100,) * 2)
        rx, ry = self.costumes[self.costume].center
        x1, y1 = self.x, self.y
        x1 -= rx - 240
        y1 += ry - 180
        y1 *= -1
        w, h = self.costumes[self.costume].size
        x2, y2 = x1 + w, y1 + h
        morph.bounds = kurt.Rectangle([x1, y1, x2, y2])
        return morph

    def save_to(self, path):
        debug("Saving sprite to %s" % path, 1)
        ensure_kurt()
        file = kurt.ScratchSpriteFile(path, load=False)
        file.stage = kurt.Stage()
        file.stage.submorphs.append(self.to_kurt())
        file.sprite.owner = file.stage
        file.save()

    def redraw(self):
        self.rerender()

    def rerender(self):
        from M30W import runtime
        runtime.stage.Refresh()

    @property
    def x(self):
        return self._x

    @x.setter
    def x(self, value):
        self._x = float(value)
        self.redraw()

    @property
    def y(self):
        return self._y

    @y.setter
    def y(self, value):
        self._y = float(value)
        self.redraw()

    @property
    def truex(self):
        return self.x + 240

    @property
    def truey(self):
        return -self.y + 180

    @property
    def pos(self):
        return '%s:%s' % (self.x, self.y)

    @pos.setter
    def pos(self, value):
        self.x, self.y = [float(x) for x in value.split(':')]
        self.redraw()

    @property
    def direction(self):
        return self._direction

    @direction.setter
    def direction(self, value):
        self._direction = float(value)
        self.rerender()

    @property
    def rotmode(self):
        return self._rotmode

    @rotmode.setter
    def rotmode(self, value):
        if not value in (NORMAL, RL, NO):
            raise TypeError("Invalid mode!")
        self._rotmode = value
        self.rerender()

    @property
    def draggable(self):
        return self._draggable

    @draggable.setter
    def draggable(self, value):
        self._draggable = bool(value)

    @property
    def size(self):
        return self._size

    @size.setter
    def size(self, value):
        self._size = float(value)
        self.rerender()

    @property
    def visible(self):
        return self._visible

    @visible.setter
    def visible(self, value):
        self._visible = bool(value)
        self.redraw()



#This file is part of the M30W software.
#Copyright (C) 2012-2013 M30W developers.
#
#This program is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program. If not, see <http://www.gnu.org/licenses/>.

from distutils.core import setup
from distutils.command.install import install
from distutils.sysconfig import get_python_lib
import sys, shutil, os, subprocess

desktop_file = \
"""[Desktop Entry]
Type=Application
Name=M30W
Comment=GUI text-based interface to Scratch (by MIT) projects.
Icon=m30w
Exec=M30W
Terminal=false
Categories=Application;Development
MimeType=application/x-scratch-project
"""

class my_install(install):
    def run(self):
        install.run(self)

        if not sys.platform == 'linux2':
            return
        message = "Detected linux. Install icons and .desktop file? (Y/n)"
        do_install = raw_input(message)
        while do_install.lower() not in ('y', 'n', ''):
            do_install = raw_input(message)
        if do_install == 'n': return

        print "Writing .desktop file..."
        with open('/usr/share/applications/m30w.desktop', 'w') as f:
            f.write(desktop_file)

        print "Copying icon..."
        icons_path = os.path.join(self.install_lib, 'M30W', 'icons')
        shutil.copyfile(os.path.join(icons_path, 'm30w.png'),
                        '/usr/share/icons/hicolor/128x128/apps/m30w.png')

        #Different commands that update icons cache and such. Tried to be as
        #cross-dist as possible
        try:
            subprocess.call(['gtk-update-icon-cache',
                             '-qf', 
                             '/usr/share/icons/hicolor'])
        except OSError:
            pass
        try:
            subprocess.call('update-desktop-database')
        except OSError:
            pass
        try:
            subprocess.call('kbuildsycoca')
        except OSError:
            pass

long_description = \
"""
M30W is a program designed to allow fast
developing of Scratch projects.
It uses a unique text syntax to allow typing of
blocks rather than laggy dragging them around.
"""

scripts = ['scripts/M30W']
if 'bdist_wininst' in sys.argv:
    scripts = ['scripts/M30W_post_install.py']

setup(name="M30W",
      version="0.2.0",
      maintainer="roijac",
      maintainer_email="roi.jacoboson1@gmail.com",
      url="http://scratch.mit.edu/forums/viewtopic.php?id=106225",
      install_requires=['kurt', 'PIL', 'wxpython'],
      packages=['M30W', 'M30W', 'M30W.compiler', 'M30W.GUI',
                'M30W.parser', 'M30W.sprites'],
      package_data={'M30W': ['icons/*', 'COPYING.txt']},
      scripts=scripts,
      description="""GUI text-based interface to Scratch (by MIT) projects.""",
      long_description=long_description,
      license="GPL3",
      cmdclass={'install': my_install})



#This file is part of the M30W software.
#Copyright (C) 2012-2013 M30W developers.
#
#This program is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program. If not, see <http://www.gnu.org/licenses/>.

"""Handles media and makes them available platform-independent.
"""
import os.path
import wx

M30W_FOLDER = os.path.split(__file__)[0]

class _Provider():
    def __init__(self, path):
        self.path = path

    def __call__(self, name, format=wx.Bitmap, ext='.png'):
        if format == wx.Bitmap:
            return wx.Bitmap(os.path.join(self.path, name + ext),
                              wx.BITMAP_TYPE_ANY)
        if format == wx.Image:
            return wx.Image(os.path.join(self.path, name + ext),
                              wx.BITMAP_TYPE_ANY)
        if format == wx.Icon:
            return wx.Icon(os.path.join(self.path, name + ext),
                              wx.BITMAP_TYPE_ANY)

get_icon = _Provider(os.path.join(M30W_FOLDER, 'icons'))



#This file is part of the M30W software.
#Copyright (C) 2012-2013 M30W developers.
#
#This program is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program. If not, see <http://www.gnu.org/licenses/>.

"""
Provides IO functions
"""
import M30W.runtime as runtime
from M30W.debug import debug
from M30W.project import Project

def save():
    runtime.project.save()

def save_as(path):
    old_path = runtime.project.path
    try:
        runtime.project.path = path
        save()
    except Exception:
        runtime.project.path = old_path
        raise

def open(path):
    old_path = runtime.project.path
    try:
        runtime.project.path = path
        reload()
    except Exception:
        runtime.project.path = old_path
        raise

def reload():
    #Referencing to old sprites, so panels won't be garbage-collected
    refs = [runtime.project.stage] + runtime.project.sprites
    runtime.project.load()
    runtime.project.changed = False
    refresh()

def new():
    runtime.project = Project()
    refresh()

def refresh():
    debug("Updating SpritePanel...", 1)
    runtime.spritePanel.UpdateList()
    debug("Done.", -1)
    debug("Refreshing stage...", 1)
    runtime.stage.Refresh()
    debug("Done.", -1)
    debug("Refreshing the notebook...", 1)
    runtime.leftPanel.RefreshPages()
    debug("Done.", -1)


#This file is part of the M30W software.
#Copyright (C) 2012-2013 M30W developers.
#
#This program is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program. If not, see <http://www.gnu.org/licenses/>.

from __future__ import division
from math import radians, degrees, hypot, atan2, sin, cos, pi
import wx, time
from M30W import runtime
from M30W.costume import _convert
from M30W.debug import debug
from M30W.sprites import NORMAL, RL


#Need to switch to canvas?
class Stage(wx.Panel):
    def __init__(self, parent):
        wx.Panel.__init__(self, parent)
        runtime.stage = self
        self.SetSizeHints(480, 360, 480, 360)
        self.SetBackgroundColour(wx.WHITE)
        self.SetSize((480, 360))
        self.Bind(wx.EVT_PAINT, self.OnPaint, self)

    def OnPaint(self, event):
        debug("Redrawing stage...", 1)
        start = time.time()
        dc = wx.PaintDC(self)
        dc.BeginDrawing()
        stage = runtime.project.stage
        dc.DrawBitmap(stage.active_costume.get_image(), 0, 0)
        for sprite in runtime.get_sprites():
            if not sprite.visible: continue
            costume = sprite.active_costume
            image = costume.get_image(wx.Image)
            if sprite.rotmode == NORMAL:
                rotation = radians(360 - (sprite.direction - 90) % 360)
                image = image.Rotate(rotation, costume.center, True)
                center = [i // 2 for i in costume.size]
                c = hypot(center[0] - costume.center[0], 
                          center[1] - costume.center[1])
                rotation += atan2(center[1] - costume.center[1],
                                  center[0] - costume.center[0])
                center = [i // 2 for i in image.GetSize()]
                x, y = center[0] + cos(rotation) * -c, center[1] + sin(rotation) * -c
    
                x = sprite.truex - x
                y = sprite.truey - y
            elif sprite.rotmode == RL:
                if sprite.direction > 180:
                    image = image.Mirror()
                x = sprite.truex - costume.center[0]
                y = sprite.truey - costume.center[1]
            else:
                x = sprite.truex - costume.center[0]
                y = sprite.truey - costume.center[1]
            dc.DrawBitmap(_convert(image, wx.Bitmap), x, y)
        dc.EndDrawing()
        debug("Done. Took %3fs" % (time.time() - start), -1)

    def GetBitmap(self):
        context = wx.ClientDC(self)
        memory = wx.MemoryDC()
        x, y = self.ClientSize
        bitmap = wx.EmptyBitmap(x, y, -1)
        memory.SelectObject(bitmap)
        memory.Blit(0, 0, x, y, context, 0, 0)
        memory.SelectObject(wx.NullBitmap)
        return bitmap
